/*
 * Interrupt request handling.
 *
 * Copyright (C) 2018, 2019, 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>
#include <l4/sys/factory.h>
#include <l4/sys/irq.h>
#include <l4/sys/rcv_endpoint.h>

#include "cap_alloc.h"
#include "irq.h"



/* IRQ operations. */

/* Create an interrupt request object. */

long ipc_create_irq(l4_cap_idx_t *irq)
{
  long err;

  *irq = ipc_cap_alloc();

  if (l4_is_invalid_cap(*irq))
    return -L4_ENOENT;

  err = l4_error(l4_factory_create_irq(l4re_env()->factory, *irq));

  if (err)
  {
    ipc_cap_free_um(*irq);
    return err;
  }

  return L4_EOK;
}

/* Initialise an interrupt request object for subsequent notifications. */

l4_msgtag_t ipc_init_irq(l4_cap_idx_t irq)
{
  return l4_irq_unmask(irq);
}

/* Prepare a thread to receive IRQ object notifications. */

long ipc_bind_irq(l4_cap_idx_t irq, l4_umword_t id, l4_cap_idx_t thread)
{
  long err = l4_error(l4_rcv_ep_bind_thread(irq, thread, id));

  if (err)
    return err;

  return L4_EOK;
}
